open Printf

exception InputError of string

let sanitize str =
    let uri = Uri.of_string str in
    match Uri.scheme uri with
    | Some scheme ->
        if scheme = "gemini" then str
        else raise (InputError ("Wrong protocol: " ^ scheme ^ "!"))
    | None -> "//" ^ str

let rec do_request ?(max_redirects=5) request = 
    if max_redirects <= 0 then print_endline "Redirect limit reached!"
    else match Gemini.send_exn @@ request with
    | Body body -> print_endline body
    | Redirect uri -> Request uri
        |> do_request ~max_redirects:(max_redirects - 1)
    | Question phrase -> begin
        printf "%s -> " phrase;
        let query = read_line () in
        match request with
        | Request uri | Answer (_, uri) ->
            do_request @@ Answer (uri, query)
    end
    | Failure (status, msg) -> printf "%d: %s\n" status msg

let () = if Array.length Sys.argv = 2 then
    let goto = sanitize (Sys.argv.(1)) in
    do_request @@ Request goto
else
    print_endline "Please specify a url!"
